package es.claro.persistence;

import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;

/**
 * 
 * @author puche
 */
public class ScopedEntityManagerFactory extends EntityManagerFactoryProxy implements LazyCloseListener {

   private final ThreadLocal<EntityManagerProxy> threadLocal;

   protected ScopedEntityManagerFactory(final EntityManagerFactory emf) {

      super(emf);
      threadLocal = new ThreadLocal<EntityManagerProxy>();
   }

   @Override
   public EntityManager createEntityManager(final Map map) {

      EntityManagerProxy em = threadLocal.get();
      if (em == null) {
         em = new LazyCloseEntityManager(super.createEntityManager(map));
         createEntityManager(em);
      }
      return em;
   }

   @Override
   public EntityManager createEntityManager() {

      EntityManagerProxy em = threadLocal.get();
      if (em == null) {
         em = new LazyCloseEntityManager(super.createEntityManager());
         createEntityManager(em);
      }
      return em;
   }

   protected void createEntityManager(final EntityManagerProxy em) {
      threadLocal.set(em);
      ((LazyCloseEntityManager) em).setLazyCloseListener(this);
   }

   protected EntityManagerProxy getEntityManager() {
      return threadLocal.get();
   }

   @Override
   public void lazilyClosed() {
      threadLocal.set(null);
   }
}